<?php

function woocommerce_accounting_export_select_options($options, $option_name){
	$value = get_option("woocommerce_accounting_{$option_name}");
	$options = apply_filters("woocommerce_accounting_options_{$option_name}", $options);
	$html = '';
	foreach ($options as $key => $label) {
			$html.= '<option value="'.esc_attr($key).'" '.selected($key, $value, true).'>'.$label.'</option>';
	}
	return $html;
}

/**
 * Format number for export
 * 
 * @var $number float
 * 
 * @return string
 */
function woocommerce_accounting_format_number($number){
	global $rounding_precision;
	$dec_separator = get_option('woocommerce_accounting_export_decimal_separator');
	$zero_opt = get_option('woocommerce_accounting_export_zero_opt');
	if(empty($number)){
		if($zero_opt != 1){
			return '';
		}
		$number = 0.00;
	}
	return str_replace('.', $dec_separator, round((float) $number, $rounding_precision));
}

/**
 * Unformat number for export
 * 
 * @var $number float
 * 
 * @return float
 */
function woocommerce_accounting_unformat_number($string){
	return (float) str_replace(',', '.', $string);
}

function woocommerce_accounting_get_tax_id_from_order_item($order_item){
	$tax_id = '';
	$taxes = $order_item->get_taxes();
	if(isset($taxes['total']) && !empty($taxes['total'])){
		foreach($taxes['total'] as $tax_id => $tax_amount){
			if(!empty($tax_amount)){
				return $tax_id;
			}
		}
	}
	return $tax_id;
}

function woocommerce_accounting_add_line($output, $line, $separator, $force=false){
	global $total_income, $total_outcome;
	$col_map = get_option('woocommerce_accounting_colorder');
	$col_outcome = $col_map['Outcome'];
	$col_income = $col_map['Income'];
	$val_income = woocommerce_accounting_unformat_number($line[$col_income]);
	$total_income += $val_income;
	$val_outcome = woocommerce_accounting_unformat_number($line[$col_outcome]);
	$total_outcome += $val_outcome;
	if(abs($val_income)+abs($val_outcome) == 0 && !$force){
		return;
	}
	if(wp_get_environment_type() == 'local'){
		echo '<tr><td>'.implode('</td><td>', $line).'</td></tr>';
		return;
	}
	fputcsv($output, $line, $separator);
}